/*
 * ICBM (IM) related stuff
 * 
 */

#include <defs.h>

int aim_sendimtoclient(struct aim_session_t *sess, struct aim_conn_t *conn,
		       char *destsn, u_int flags, char *msg)
{ 
  struct command_tx_struct tx;
  struct aimd_clientinfo *client = NULL;
  struct aim_tlvlist_t *tlvlist = NULL;
  int i = 0, y, z;
  char *msgblock;

  client = aimd_findclient(destsn);
  if (!client) {
    PROF_COUNTERINC(prof_msgroutefails);
    printf("destination %s not online!\n", destsn);
    /* XXX: send error */
    return 0;
  }

  tx.conn = aim_getconn_type(client->sess, AIM_CONN_TYPE_BOS);

  tx.commandlen = 512 + strlen(msg); /* arbitrarily large */
  tx.data = malloc(tx.commandlen);
  memset(tx.data, 0x00, tx.commandlen);
  
  tx.lock = 1;
  tx.type = 0x02;

  msgblock = malloc(strlen(msg)+128); /* ?? */
  memset(msgblock, 0, strlen(msg)+128);

  z = 0;
  z += aimutil_put8(msgblock+z, 0x00);
  z += aimutil_put8(msgblock+z, 0x00);
  
  z += aimutil_put16(msgblock+z, 0x02);
  z += aimutil_put8(msgblock+z, 0x00);
  z += aimutil_put8(msgblock+z, 0x00);

  z += aimutil_put8(msgblock+z, 0x00);
  z += aimutil_put8(msgblock+z, 0x00);

  z += aimutil_put16(msgblock+z, strlen(msg)+4);
  
  /* flag words */
  z += aimutil_put16(msgblock+z, 0x0000);
  z += aimutil_put16(msgblock+z, 0x0000);

  /* msg */
  z += aimutil_putstr(msgblock+z, msg, strlen(msg));

  /*
   * SNAC header 
   */
  i = 0;
  i += aimutil_put16(tx.data+i, 0x0004);
  i += aimutil_put16(tx.data+i, 0x0007);
  i += aimutil_put16(tx.data+i, 0x0000);
  i += aimutil_put16(tx.data+i, 0x0000);
  i += aimutil_put16(tx.data+i, 0x0000);
  
  /*
   * Message cookie
   */
  for (y=0;y<8;y++)
    i += aimutil_put8(tx.data+i+y, (u_char) random());

  /*
   * Channel ID
   */
  i += aimutil_put16(tx.data+i, 0x0001);

  /*
   * Source SN
   */
  i += aimutil_put8(tx.data+i, strlen(sess->logininfo.screen_name));
  i += aimutil_putstr(tx.data+i, sess->logininfo.screen_name, strlen(sess->logininfo.screen_name));

  /*
   * Warning level
   */
  i += aimutil_put16(tx.data+i, 0x0000);

  /* class */
  aim_addtlvtochain16(&tlvlist, 0x0001, AIM_CLASS_FREE | AIM_CLASS_TRIAL);

  /* member-since date */
  aim_addtlvtochain32(&tlvlist, 0x0002, time(NULL));

  /* on-since date */
  aim_addtlvtochain32(&tlvlist, 0x0003, time(NULL));

  /* idle-time */
  aim_addtlvtochain16(&tlvlist, 0x0004, 0);

  /* session length (AIM) */
  aim_addtlvtochain16(&tlvlist, 0x000f, 0);

  /* add msgblock to chain */
  aim_addtlvtochain_str(&tlvlist, 0x0002, msgblock, z);

  i += aimutil_put16(tx.data+i, aim_counttlvchain(&tlvlist));
  tx.commandlen = aim_writetlvchain(tx.data+i, tx.commandlen-i, &tlvlist)+i;
  
  free(msgblock);
  aim_freetlvchain(&tlvlist);

  tx.lock = 0;

  /*
   * Because we're directly injecting packets into another
   * threads' transmit queue, we have to lock it.  However, 
   * sending to ourselves is a special case, since we not
   * only don't want to lock our own queue, but we must not.
   * If we do, we sit around and wait for ourselves to do 
   * something while we're waiting for ourselves.  Deadlock.
   */
  if (client->sess == sess) {
    aim_tx_enqueue(sess, &tx);
  } else {
    pthread_mutex_lock(&((struct aimd_sessdata *)client->sess->aux_data)->txqueuelock);	
    aim_tx_enqueue(client->sess, &tx);
    pthread_mutex_unlock(&((struct aimd_sessdata *)client->sess->aux_data)->txqueuelock);
  }

  PROF_COUNTERINC(prof_msgroutes);

  return 0;
}

int aimd_handleicbm(struct aim_session_t *sess, struct command_rx_struct *cur)
{
  int i = 0;
  unsigned char icbmcookie[8];
  unsigned short chanid;
  int destsnlen;
  char destsn[MAXSNLEN+1];
  struct aim_tlvlist_t *tlvlist = NULL;
  char *msgblock = NULL;
  int flags = 0;

  i += 10;

  for(; i < 18; i++)
    icbmcookie[i-10] = aimutil_get8(cur->data+i);

  chanid = aimutil_get16(cur->data+i);
  i += 2;

  destsnlen = aimutil_get8(cur->data+i++);
  strcpy(destsn, cur->data+i);
  i += destsnlen;

  tlvlist = aim_readtlvchain(cur->data+i, cur->commandlen-i);

  if (chanid == 1) {
    if (aim_gettlv(tlvlist, 0x0002, 1)) {
      msgblock = aim_gettlv_str(tlvlist, 0x0002, 1);
    } else {
      printf("no message block!\n");
    }
    if (aim_gettlv(tlvlist, 0x0003, 1)) {
      flags |= AIM_IMFLAGS_ACK;
    }
    if (aim_gettlv(tlvlist, 0x0004, 1)) {
      flags |= AIM_IMFLAGS_AWAY;
    }
    
#ifdef PROFILING
#define PROF_SPRINTVAL(x) { \
  pthread_mutex_lock(&x.lock); \
  m += sprintf(profmsg+m, "%Ld (%G/sec)", x.val, ((double)x.val/(double)elapsed)); \
  pthread_mutex_unlock(&x.lock); \
}
    if (strcmp(msgblock+0x0d, "profstats") == 0) {
      char profmsg[1152];
      time_t elapsed;
      int m = 0;

      elapsed = time(NULL) - prof_starttime;

      m += sprintf(profmsg+m, "<br>---- Profiling info ----<br>");
      m += sprintf(profmsg+m, "Connection count: ");
      PROF_SPRINTVAL(prof_conncount);
      m += sprintf(profmsg+m, "<br>Packet rx count: ");
      PROF_SPRINTVAL(prof_packrecv);
      m += sprintf(profmsg+m, "<br>Packet tx count: ");
      PROF_SPRINTVAL(prof_packtrans);
      m += sprintf(profmsg+m, "<br>Message routes completed: ");
      PROF_SPRINTVAL(prof_msgroutes);
      m += sprintf(profmsg+m, "<br>Failed message routes: ");
      PROF_SPRINTVAL(prof_msgroutefails);
      m += sprintf(profmsg+m, "<br>Time elapsed: %dsec", (int)elapsed);
      m += sprintf(profmsg+m, "<br>---- END ----");
      TXCALLLOCKED(aim_sendimtoclient(sess, cur->conn, sess->logininfo.screen_name, 0, profmsg););
#undef PROF_PRINTVAL
    } else
#endif
      TXCALLLOCKED(aim_sendimtoclient(sess, cur->conn, destsn, flags, msgblock+0x0d););

    free(msgblock);
    
  } else {
    printf("unknown channel id %x\n", chanid);
  }
  
  aim_freetlvchain(&tlvlist);

  return 1;
}
