/* mfaim
 * Copyright (c) 1999 Josh Myer <josh@joshisanerd.com>
 * console-mode AIM client
 *
 * mfaim is based on naim;
 * naim is Copyright (c) 1998, 1999 Daniel Reed <n@ml.org>
 * 
 * mfaim also uess libfaim,
 * FAIM is Copyright 1998 Adam Fritzler <afritz@iname.com>
 * FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
 */

#include <mfaim.h>
#include <sys/stat.h>

extern WINDOW *win_status;
extern buddywin_t nullbwin;
extern faimconf_t faimconf;
extern struct aim_conn_t *nfaim_conn;
extern struct aim_session_t mfaim_session;
time_t idlesince = 0;

void logim(char *source, char *target, char *msg) {
  static FILE *logfile;
  time_t t = 0;

  if (faimconf.nolog == 1)
    return;
  t = time(NULL);

  if ((source == NULL) && (target == NULL) && (msg != NULL)
      && (*msg != 0)) {
    struct tm *tmptr = NULL;

    tmptr = localtime(&t);
    if ((logfile = fopen(msg, "a")) == NULL)
      printf("Unable to open \"%s\": %m", msg);
    fchmod(fileno(logfile), 0600);
    fprintf(logfile, "\n*** Log opened %02i-%02i-%04i at"
	    " %02i:%02i.\n", tmptr->tm_mon+1, tmptr->tm_mday,
	    1900+tmptr->tm_year, tmptr->tm_hour,
	    tmptr->tm_min);
    return;
  }
  fprintf(logfile, "%lu %10s | %10s | %s\n", t, source, target, msg);
  fflush(logfile);
}

void mfaim_send_im(char *SN, char *msg) {
  if (aim_send_im(&mfaim_session, nfaim_conn, SN, 0, msg) == -1) {
    buddywin_t *bwin;
    if ((bwin = wgetwin(SN)) == &nullbwin)
      printf("Error sending IM to %s\n", SN);
    else 
      hwprintf(bwin->win, IMWIN_TEXT_U, "Error sending IM -- too large?<br>");
  }
}

void mfaim_send_im_away(char *SN, char *msg) {
  buddywin_t *bwin;

  if ((bwin = wgetwin(SN)) == &nullbwin)
    printf("Sent away IM to %s (%s)\n", SN, msg);
  else {
    if (atoi(secs_getvar("timestamps")) & 0x02)
      mfaim_dprintf(bwin->win, IMWIN_TEXT_U, "<B>%s</B> is away: %s<br>",mfaim_session.logininfo.screen_name, msg);
    else 
      hwprintf(bwin->win, IMWIN_TEXT_U, "<B>%s</B> is away: %s<br>",mfaim_session.logininfo.screen_name, msg);
  }

  if (aim_send_im(&mfaim_session, nfaim_conn, SN, AIM_IMFLAGS_AWAY, msg) == -1) {
    if (bwin == &nullbwin)
      printf("Error sending away IM to %s\n", SN);
    else 
      hwprintf(bwin->win, IMWIN_TEXT_U, "Error sending away IM -- too large?<br>");
  }
}

void setaway(void) {
  nullbwin.away = 1;
  aim_bos_setprofile(&mfaim_session, nfaim_conn, faimconf.profile,secs_getvar("awaymsg"), AIM_CAPS_CHAT);
  idlesince = time(NULL);
  printf("You are now away--hurry back!\n");
  return;
}

void unsetaway(void) {
  nullbwin.away = 0;
  idlesince = 0;
  aim_bos_setprofile(&mfaim_session, nfaim_conn, faimconf.profile, NULL, AIM_CAPS_CHAT);
  printf("You are no longer away--welcome back =)\n");
  return;
}

void sendaway(char *SN) {
  char buf[1124];
  struct tm *tmptr = NULL;

  if (nullbwin.away == 0)
    return;

  tmptr = localtime(&idlesince);
  snprintf(buf, sizeof(buf), "[Away since %02i/%02i at %02i:%02i] %s",
	   1+tmptr->tm_mon, tmptr->tm_mday, tmptr->tm_hour,
	   tmptr->tm_min, secs_getvar("awaymsg"));
  mfaim_send_im_away(SN, buf);
}
