/* mfaim
 * Copyright (c) 1999 Josh Myer <josh@joshisanerd.com>
 * console-mode AIM client
 *
 * mfaim is based on naim;
 *  naim is Copyright (c) 1998, 1999 Daniel Reed <n@ml.org>
 * 
 * mfaim also uess libfaim,
 *  FAIM is Copyright 1998 Adam Fritzler <afritz@iname.com>
 *  FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
 */

#include <mfaim.h>
#include <signal.h>

extern WINDOW	*win_input, *win_buddy, *win_display;
extern buddywin_t	*curbwin, nullbwin;
extern faimconf_t	faimconf;
extern int	wsetup_called;
int	continuemainloop = 1, stayconnected;
mfaimses_t      mfaimses;
char *faimrcfile = NULL;
struct aim_session_t mfaim_session;

int	main(int argc, char **args) {
  struct timeval	timeout;
  time_t	lastcycle = 0;
  char	buf[256];
  int selstat = -1;
  struct aim_conn_t	*cur_conn;
  struct aim_conn_t *stdinconn;

#if 1
  /* we can just advertise Version 5 now and be ahead for a while :) */
  struct client_info_s info = {"mfaim-pre2; libfaim 0.95", 5, 0, 4242, "us", "en"};
#else
  struct client_info_s info = {"AOL Instant Messenger (SM). version 3.5.1670/WIN32", 3, 5, 686, "us", "en"};
#endif

  int curopt = 0;

  nullbwin.away = 0;

  while ((curopt = getopt(argc, args, "c:f:vh")) != EOF) {
    switch (curopt) {
    case 'c': 
    case 'f':
      faimrcfile = optarg;
      if (!faimrcfile || (faimrcfile[0] == '-')) {
	printf("-%c option requires filename\n",curopt);
	exit(-1);
      }
      break;
    case 'v':
      printf("%s\n", info.clientstring);
      exit(0);
    case 'h':
    default:
      printf("%s\n", info.clientstring);
      printf("Usage:\n");
      printf("\t-c filename\tUse filename as faimrc (defaults to ~/.faimrc)\n");
      printf("\t-f filename\tUse filename as faimrc (defaults to ~/.faimrc)\n");
      printf("\t-v\t\tPrint version information\n");
      printf("\n");
      exit(0);
    }
  }
	
  if (!faimrcfile) {
    snprintf(buf, sizeof(buf), "%s/.faimrc", getenv("HOME"));
    faimrcfile = buf;
  }
		
  secs_init();
  initscr();
    
  wsetup_called = 2;

  printf("Reading %s\n", faimrcfile);
  if (mfaim_read_config(faimrcfile) == 0) {
    wshutitdown();
    printf("Error %i reading %s: %s\n", errno, faimrcfile, 
	   strerror(errno));
    exit(-1);
  }
  endwin();
  wsetup_called = 0;
  wsetup();
  bsetup();
  gotkey(0);	// initialize gotkey() buffer
  wcols(&faimconf);
  if (faimconf.nolog == 0) {
    if (*(faimconf.logfile) == 0)
      strncpy(faimconf.logfile, "mfaim.log",
	      sizeof(faimconf.logfile)-1);
    printf("Opening log file (%s)\n", faimconf.logfile);
    logim(NULL, NULL, faimconf.logfile);
  } else if (*(faimconf.logfile) != 0)
    printf("You have logging disabled, but you specified a log file.\n");

  printf("Contact Josh Myer <josh@joshisanerd.com> if you'd\n");
  printf("like information about mfaim.\n");
  printf("This Program is based on naim, by Daniel Reed.\n");

  wrefresh(nullbwin.win);
  wrefresh(win_input);
	
  mfaimses.online = mfaimses.connected = 0;
  do {
    aim_session_init(&mfaim_session);
    printf("Authenticating %s.\n", faimconf.screenname);

    wrefresh(nullbwin.win);

    if (!(stdinconn = aim_conn_getnext(&mfaim_session))) {
      wrefresh(nullbwin.win);
      wshutitdown();
      printf("Unable to allocate AIM structure for stdin\n");
      exit(1);
    }
    stdinconn->fd = STDIN_FILENO;

    if ((cur_conn = aim_newconn(&mfaim_session,AIM_CONN_TYPE_AUTH, 
				strlen(faimconf.aimserver)?
				faimconf.aimserver:
				FAIM_LOGIN_SERVER)) == NULL) {
      wrefresh(nullbwin.win);
      wshutitdown();
      printf("FAIM was unable to allocate AIM structure for Authorizer connection.\n");
      exit(1);
    } else if (cur_conn->fd == -1) {
      if (cur_conn->status & AIM_CONN_STATUS_RESOLVERR) {
	wrefresh(nullbwin.win);
	wshutitdown();
	printf("FAIM was unable to resolve"
	       " authorizer name.\n");
      } else if (cur_conn->status & AIM_CONN_STATUS_CONNERR) {
	wrefresh(nullbwin.win);
	wshutitdown();
	printf("FAIM was unable to connect to"
	       " authorizer.\n");
      } else {
	wrefresh(nullbwin.win);
	wshutitdown();
	printf("FAIM was unable to connect to"
	       " authorizer. (Unknown error.)\n");
      }
      aim_conn_kill(&mfaim_session, &cur_conn);
      aim_conn_kill(&mfaim_session, &stdinconn);
      exit(1);
    } 

    aim_conn_addhandler(&mfaim_session, cur_conn,
			AIM_CB_FAM_SPECIAL, AIM_CB_SPECIAL_AUTHSUCCESS, 
			mfaim_auth_success, 0);
    aim_send_login(&mfaim_session, cur_conn, faimconf.screenname,
		   faimconf.password, &info);
    mfaimses.lag = 0;
    mfaimses.sn[MAXSNLEN+1] = '\0';
    strncpy(mfaimses.sn, faimconf.screenname, MAXSNLEN);
  
    /* Ignore break. */
    signal(SIGINT, SIG_IGN);
		
    /* We should catch this and do a real redraw */
    /* signal(SIGWINCH, mfaim_winch); */

    timeout.tv_sec = 0;
    timeout.tv_usec = 100;

    debug_printf("Entering event loop.\n");
    stayconnected = 1;
    /* this is the main loop */

    lastcycle = time(NULL);

    while (stayconnected) {
      static time_t now;

      cur_conn = aim_select(&mfaim_session, &timeout, &selstat);

      now = time(NULL);
      now -= now % 60;
  
      /* so what if the queues arent in the same state we left them.. */
      if ((now - lastcycle) >= 60)
	event_handle(lastcycle = now);

      switch (selstat) {
      case -1: /* error in select */
	printf("error in aim_select\n"); /* XXX: what should we do here? */
      case 0: /* timeout -- nothing happened */
	break;

      case 1: /* outgoing data waiting */
	aim_tx_flushqueue(&mfaim_session);
	break;

      case 2: /* incoming data waiting */
	if (cur_conn == stdinconn) 
	  gotkey(getch());
	else {
	  if (aim_get_command(&mfaim_session, cur_conn) >= 0)
	    aim_rxdispatch(&mfaim_session); 
	  else /* there was a problem in get_command */
	    stayconnected = 0;
	}
	break;

      default:
	break; /* invalid */
      }
      
      wnoutrefresh(win_buddy);
      wnoutrefresh(win_display);
      wnoutrefresh(curbwin->win);
      wnoutrefresh(win_input);
      doupdate();
      timeout.tv_sec = 0;
      timeout.tv_usec = 100;
    }

    mfaimses.connected = 0;
    printf("Connection to Oscar lost, attempting to reconnect.\n");
    bclearall();
    wnoutrefresh(win_buddy);
    wnoutrefresh(nullbwin.win);
    wnoutrefresh(win_input);
    doupdate();
    sleep(1);
    printf("Closing all lingering AIM connection.\n");
    aim_logoff(&mfaim_session);
    sleep(1);
  } while (continuemainloop);

  aim_logoff(&mfaim_session);
  wshutitdown();
  exit(0);
}
