/* mfaim
 * Copyright (c) 1999 Josh Myer <josh@joshisanerd.com>
 * console-mode AIM client
 *
 * mfaim is based on naim;
 *  naim is Copyright (c) 1998, 1999 Daniel Reed <n@ml.org>
 * 
 * mfaim also uess libfaim,
 *  FAIM is Copyright 1998 Adam Fritzler <afritz@iname.com>
 *  FAIM uses code Copyright 1998 Brock Wilcox <awwaiid@iname.com>
 */

#include <mfaim.h>

static sn_queue_t *sn_queue_root = NULL;

int aimcmp(char *sn1, char *sn2) {
  register char	*s1 = sn1, *s2 = sn2;
  if (sn1 == NULL || sn2 == NULL)
    return(-1);
  
  while ((*s1 != 0) && (*s2 != 0)) {
    while (isspace(*s1))
      s1++;
    while (isspace(*s2))
      s2++;
    if (toupper(*s1) > toupper(*s2))
      return(1);
    else if (toupper(*s1) < toupper(*s2))
      return(-1);
    s1++;
    s2++;
  }
  if (*s1 != 0)
    return(-1);
  else if (*s2 != 0)
    return(1);
  return(0);
}

/* sn_normalize is designed such that to and SN can be the same memory
   location; y is assured to be < x. */

char * sn_normalize(char *to, char *from) { 
  unsigned char x, y;
  
  for(x = y = 0; x < strlen(from); y++, x++) {
    while(isspace(from[x]))
      x++;
    to[y] = toupper(from[x]);
  }
  
  while(y < x)/* terminating zero and clean up */
    to[y++] = 0;
  
  return(to);
}
char * cr_normalize(char *to, char *from) { 
  unsigned char x, y;
  
  for(x = y = 0; x < strlen(from); y++, x++) {
    while(isspace(from[x]))
      x++;
    to[y] = toupper(from[x]);
  }
  
  while(y < x)/* terminating zero and clean up */
    to[y++] = 0;
  
  return(to);
}


void sn_queue_enq(char *SN) {
  sn_queue_t *snq;

  snq = (sn_queue_t *)malloc(sizeof(sn_queue_t));

  sn_queue_rm(SN);

  memcpy(snq->screenname, SN,MAXWLEN+1);

  if (sn_queue_root == NULL)
    snq->index = 0;
  else 
    snq->index = sn_queue_root->index + 1;

  snq->next = sn_queue_root;
  sn_queue_root = snq;

  return;
}
  

char *sn_queue_deq(char *sn) {

  if (sn_queue_peek() == NULL) 
    return(NULL);

  memcpy(sn, sn_queue_peek(), MAXWLEN+1);

  sn_queue_rm(sn);
  return(sn);
}

char *sn_queue_peek(void) {
  sn_queue_t *snq;

  snq = sn_queue_root;

  if (snq == NULL) 
    return NULL;

  while(snq->next != NULL)
    snq = snq->next;

  return(snq->screenname);
}

void sn_queue_rm(char *SN) {

  sn_queue_t *snq, *snqb, *snqc;

  if(sn_queue_root == NULL)
    return;  

  if (aimcmp(sn_queue_root->screenname, SN) == 0) {
    free(sn_queue_root->screenname);
    sn_queue_root = sn_queue_root->next;
    return;
  }

  if(sn_queue_root->next == NULL) 
    return;

  snqb = sn_queue_root;
  snq = snqb->next;
  while(snq != NULL) {
    snq = snqb->next;
    if(aimcmp(snq->screenname,SN) == 0) {
      snqb->next = snq->next;
      if(snq->next != NULL) {
	snqc = sn_queue_root;
	while(snqc != snqb) {
	  snqc->index--;
	  snqc = snqc->next;
	}
      }
      snqb->index--;
      free(snq->screenname);
      return;
    }
    snqb = snq;
  }
  
  return;
}
